@@ -6,12 +6,14 @@
 #' and splits the data based on column name prefixes.
 #'
 #' @param qualtrics_alias Character string specifying the Qualtrics survey alias to retrieve.
+#' @param institution Character string; default NULL, specify location
 #' @param label Logical; default TRUE, returns coded values as labels instead of raw values.
+#' @param interview_date Logical or Date String, returns all data before date
 #' @param lower default TRUE convert prefixes to lower case
 #'
 #' @return Creates multiple dataframes in the global environment, one for each survey
 #'   detected in the data. Each dataframe is named after its survey prefix.
-#'   
+#'
 #' @details
 #' The function performs the following steps:
 #' \itemize{
@@ -26,7 +28,7 @@
 #' \dontrun{
 #' # Parse a Qualtrics export containing multiple surveys
 #' qualtrics.rune("combined_surveys", label = FALSE)
-#' 
+#'
 #' # After running, access individual survey dataframes directly:
 #' head(pss)  # Access the PSS survey dataframe
 #' head(cesd) # Access the CESD survey dataframe
@@ -34,71 +36,71 @@
 #'
 #' @importFrom dplyr filter select
 #' @export
-qualtrics.rune <- function(qualtrics_alias, label = FALSE, lower = TRUE){
-  
-  df <- qualtrics(qualtrics_alias, label)
-  
+qualtrics.rune <- function(qualtrics_alias, institution = NULL, label = FALSE, interview_date = NULL, lower = TRUE){
+
+  df <- qualtrics(qualtrics_alias, institution, label, interview_date)
+
   # Define potential identifiers
   identifiers <- c("participantId", "workerId", "PROLIFIC_PID", "src_subject_id")
-  
+
   # Filter to keep only existing keys in the dataframe
   existing_keys <- identifiers[identifiers %in% names(df)]
   # Check if any identifiers exist in the dataframe
   if (length(existing_keys) == 0) {
     stop("No valid identifiers found in the dataframe.")
   }
-  
+
   # Print existing identifiers for debugging
   #print("Existing identifiers:")
   #print(existing_keys)
-  
+
   # Find the first identifier with non-NA values
   identifier <- NA
   for (key in existing_keys) {
     non_na_count <- sum(!is.na(df[[key]]))
-    
+
     # Debug print to check how many non-NA values exist in each column
     #print(paste("Checking identifier:", key, "with", non_na_count, "non-NA values"))
-    
+
     if (non_na_count > 0) {  # As long as there's at least 1 non-NA value
       identifier <- key
       break
     }
   }
-  
+
   # If no column has any non-NA values, issue a warning or stop
   if (is.na(identifier)) {
     stop(paste("No identifier found without NA values or multiple identifiers exist:", existing_keys))
   }
-  
+
   # Print the detected identifier for debugging
   message(paste("Detected identifier:", identifier))
-  
+
   # Define common columns to include if they exist
   common_columns <- c(
     "record_id", "subjectkey", "site", "phenotype", "visit", "week",
-    "state", "status", "lost_to_followup", "lost_to_follow-up", 
+    "state", "status", "lost_to_followup", "lost_to_follow-up",
     "interview_age", "interview_date"
   )
-  
+
   # Filter to only include common columns that exist in the original dataframe
   common_columns_exist <- common_columns[common_columns %in% names(df)]
-  
+
   # Exclude non-survey and specific columns from survey prefix detection
   non_survey_columns <- c(existing_keys, common_columns)
   survey_columns <- names(df)[!names(df) %in% non_survey_columns & grepl("_", names(df))]
-  
+
   # Extract unique survey prefixes from survey-specific column names
   extract_first_part <- function(string) {
     parts <- strsplit(string, "_")[[1]]
     return(parts[1])
   }
   survey_prefixes <- unique(sapply(survey_columns, extract_first_part))
-  
+
   # Exclude prefixes that might still be problematic
   excluded_prefixes <- c("PROLIFIC", "interview")
   survey_prefixes <- survey_prefixes[!survey_prefixes %in% excluded_prefixes]
-  
+
   # Create a list of dataframes, one for each survey
   output = list()
   for (prefix in survey_prefixes) {
@@ -107,23 +109,25 @@ qualtrics.rune <- function(qualtrics_alias, label = FALSE, lower = TRUE){
       # Create subset dataframe with identifier, common columns, and survey-specific columns
       all_columns <- c(identifier, common_columns_exist, survey_specific_columns)
       subset_df <- df[, all_columns, drop = FALSE]
-      
+
       # Apply lowercase transformation if requested
       if (lower) {
         # Always preserve the identifier column name and common columns
         cols_to_transform <- !names(subset_df) %in% c(identifier, common_columns_exist)
         names(subset_df)[cols_to_transform] <- tolower(names(subset_df)[cols_to_transform])
       }
-      
+
       # Add to output list with lowercase prefix as key
       output[[tolower(prefix)]] <- subset_df
     }
   }
-  
+
   names(output) <- tolower(survey_prefixes)
-  
+
   # Use parent.frame() instead of globalenv() for CRAN compliance
   list2env(output, parent.frame())
+  # Add an explicit return of invisible(NULL) to suppress output
+  invisible(NULL)
 }
 
 #' Parse composite MongoDB collection into component data frames by variable prefix
@@ -139,7 +143,7 @@ qualtrics.rune <- function(qualtrics_alias, label = FALSE, lower = TRUE){
 #'
 #' @return Creates multiple dataframes in the global environment, one for each survey
 #'   detected in the data. Each dataframe is named after its survey prefix.
-#'   
+#'
 #' @details
 #' The function performs the following steps:
 #' \itemize{
@@ -154,7 +158,7 @@ qualtrics.rune <- function(qualtrics_alias, label = FALSE, lower = TRUE){
 #' \dontrun{
 #' # Parse a Qualtrics export containing multiple surveys
 #' mongo.rune("combined_surveys", label = FALSE)
-#' 
+#'
 #' # After running, access individual survey dataframes directly:
 #' head(pss)  # Access the PSS survey dataframe
 #' head(cesd) # Access the CESD survey dataframe
@@ -163,71 +167,71 @@ qualtrics.rune <- function(qualtrics_alias, label = FALSE, lower = TRUE){
 #' @importFrom dplyr filter select
 #' @export
 mongo.rune <- function(collection, db_name = NULL, lower = TRUE ){
-  
+
   df <- mongo(collection, db_name)
-  
+
   # Define potential identifiers
   identifiers <- c("participantId", "workerId", "PROLIFIC_PID", "src_subject_id")
-  
+
   # Filter to keep only existing keys in the dataframe
   existing_keys <- identifiers[identifiers %in% names(df)]
-  
+
   # Check if any identifiers exist in the dataframe
   if (length(existing_keys) == 0) {
     stop("No valid identifiers found in the dataframe.")
   }
-  
+
   # Print existing identifiers for debugging
   #print("Existing identifiers:")
   #print(existing_keys)
-  
+
   # Find the first identifier with non-NA values
   identifier <- NA
   for (key in existing_keys) {
     non_na_count <- sum(!is.na(df[[key]]))
-    
+
     # Debug print to check how many non-NA values exist in each column
     #print(paste("Checking identifier:", key, "with", non_na_count, "non-NA values"))
-    
+
     if (non_na_count > 0) {  # As long as there's at least 1 non-NA value
       identifier <- key
       break
     }
   }
-  
+
   # If no column has any non-NA values, issue a warning or stop
   if (is.na(identifier)) {
     stop(paste("No identifier found without NA values or multiple identifiers exist:", existing_keys))
   }
-  
+
   # Print the detected identifier for debugging
   message(paste("Detected identifier:", identifier))
-  
+
   # Define common columns to include if they exist
   common_columns <- c(
     "record_id", "subjectkey", "site", "phenotype", "visit", "week",
-    "state", "status", "lost_to_followup", "lost_to_follow-up", 
+    "state", "status", "lost_to_followup", "lost_to_follow-up",
     "interview_age", "interview_date"
   )
-  
+
   # Filter to only include common columns that exist in the original dataframe
   common_columns_exist <- common_columns[common_columns %in% names(df)]
-  
+
   # Exclude non-survey and specific columns from survey prefix detection
   non_survey_columns <- c(existing_keys, common_columns)
   survey_columns <- names(df)[!names(df) %in% non_survey_columns & grepl("_", names(df))]
-  
+
   # Extract unique survey prefixes from survey-specific column names
   extract_first_part <- function(string) {
     parts <- strsplit(string, "_")[[1]]
     return(parts[1])
   }
   survey_prefixes <- unique(sapply(survey_columns, extract_first_part))
-  
+
   # Exclude prefixes that might still be problematic
   excluded_prefixes <- c("PROLIFIC", "interview")
   survey_prefixes <- survey_prefixes[!survey_prefixes %in% excluded_prefixes]
-  
+
   # Create a list of dataframes, one for each survey
   output = list()
   for (prefix in survey_prefixes) {
@@ -236,23 +240,148 @@ mongo.rune <- function(collection, db_name = NULL, lower = TRUE ){
       # Create subset dataframe with identifier, common columns, and survey-specific columns
       all_columns <- c(identifier, common_columns_exist, survey_specific_columns)
       subset_df <- df[, all_columns, drop = FALSE]
-      
+
       # Apply lowercase transformation if requested
       if (lower) {
         # Always preserve the identifier column name and common columns
         cols_to_transform <- !names(subset_df) %in% c(identifier, common_columns_exist)
         names(subset_df)[cols_to_transform] <- tolower(names(subset_df)[cols_to_transform])
       }
-      
+
       # Add to output list with lowercase prefix as key
       output[[tolower(prefix)]] <- subset_df
     }
   }
-  
+
   names(output) <- tolower(survey_prefixes)
-  
+
   # Use parent.frame() instead of globalenv() for CRAN compliance
   list2env(output, parent.frame())
+  # Add an explicit return of invisible(NULL) to suppress output
+  invisible(NULL)
+}
+
+#' Parse composite REDCap instrument into component data frames by variable prefix
+#'
+#' This function fetches a REDCap instrument and separates it into individual data frames
+#' for each survey/collection detected in the data based on column name prefixes.
+#' It identifies the appropriate identifier column and splits the data accordingly.
+#'
+#' @param instrument_name Name of the REDCap instrument
+#' @param raw_or_label Whether to return raw or labeled values
+#' @param redcap_event_name Optional event name filter
+#' @param batch_size Number of records to retrieve per batch
+#' @param records Optional vector of specific record IDs
+#' @param fields Optional vector of specific fields
+#' @param exclude_pii Default TRUE remove all fields marked as identifiable
+#' @param interview_date Optional; date filtering parameter
+#' @param date_format Default ymd define date format for interview_date
+#' @param lower default TRUE convert prefixes to lower case
+#'
+#' @return Creates multiple dataframes in the parent environment, one for each survey
+#'   detected in the data. Each dataframe is named after its survey prefix.
+#'
+#' @export
+redcap.rune <- function(instrument_name, raw_or_label = "raw",
+                        redcap_event_name = NULL, batch_size = 1000,
+                        records = NULL, fields = NULL, exclude_pii = TRUE,
+                        interview_date = NULL, date_format = "ymd", lower = TRUE) {
+  # Fetch the data using redcap()
+  df <- redcap(instrument_name, raw_or_label, redcap_event_name, batch_size,
+               records, fields, exclude_pii, interview_date, date_format)
+
+  # Define potential identifiers
+  identifiers <- c("record_id", "participantId", "workerId", "PROLIFIC_PID", "src_subject_id")
+
+  # Filter to keep only existing keys in the dataframe
+  existing_keys <- identifiers[identifiers %in% names(df)]
+
+  # Check if any identifiers exist in the dataframe
+  if (length(existing_keys) == 0) {
+    stop("No valid identifiers found in the dataframe.")
+  }
+
+  # Find the first identifier with non-NA values
+  identifier <- NA
+  for (key in existing_keys) {
+    non_na_count <- sum(!is.na(df[[key]]))
+    if (non_na_count > 0) {  # As long as there's at least 1 non-NA value
+      identifier <- key
+      break
+    }
+  }
+
+  # If no column has any non-NA values, issue a warning or stop
+  if (is.na(identifier)) {
+    stop(paste("No identifier found without NA values or multiple identifiers exist:", existing_keys))
+  }
+
+  # Print the detected identifier for debugging
+  message(paste("Detected identifier:", identifier))
+
+  # Define common columns to include if they exist
+  common_columns <- c(
+    "record_id", "subjectkey", "site", "phenotype", "visit", "week",
+    "state", "status", "lost_to_followup", "lost_to_follow-up",
+    "interview_age", "interview_date", "redcap_event_name"
+  )
+
+  # Filter to only include common columns that exist in the original dataframe
+  common_columns_exist <- common_columns[common_columns %in% names(df)]
+
+  # Exclude non-survey and specific columns from survey prefix detection
+  non_survey_columns <- c(existing_keys, common_columns)
+  survey_columns <- names(df)[!names(df) %in% non_survey_columns & grepl("_", names(df))]
+
+  # Extract unique survey prefixes from survey-specific column names
+  extract_first_part <- function(string) {
+    parts <- strsplit(string, "_")[[1]]
+    return(parts[1])
+  }
+  survey_prefixes <- unique(sapply(survey_columns, extract_first_part))
+
+  # Exclude prefixes that might still be problematic
+  excluded_prefixes <- c("PROLIFIC", "interview", "redcap")
+  survey_prefixes <- survey_prefixes[!survey_prefixes %in% excluded_prefixes]
+
+  # Create a list of dataframes, one for each survey
+  output = list()
+  for (prefix in survey_prefixes) {
+    survey_specific_columns <- grep(paste0("^", prefix, "_"), names(df), value = TRUE)
+    if (length(survey_specific_columns) > 0) {
+      # Create subset dataframe with identifier, common columns, and survey-specific columns
+      all_columns <- c(identifier, common_columns_exist, survey_specific_columns)
+      # Make sure all columns exist in df
+      all_columns <- all_columns[all_columns %in% names(df)]
+      subset_df <- df[, all_columns, drop = FALSE]
+
+      # Apply lowercase transformation if requested
+      if (lower) {
+        # Always preserve the identifier column name and common columns
+        cols_to_transform <- !names(subset_df) %in% c(identifier, common_columns_exist)
+        names(subset_df)[cols_to_transform] <- tolower(names(subset_df)[cols_to_transform])
+      }
+
+      # Add to output list with lowercase prefix as key if requested
+      if (lower) {
+        output[[tolower(prefix)]] <- subset_df
+      } else {
+        output[[prefix]] <- subset_df
+      }
+    }
+  }
+
+  if (lower) {
+    names(output) <- tolower(survey_prefixes)
+  } else {
+    names(output) <- survey_prefixes
+  }
+
+  # Use parent.frame() instead of globalenv() for CRAN compliance
+  list2env(output, parent.frame())
+
+  # Add an explicit return of invisible(NULL) to suppress output
+  invisible(NULL)
 }
 
 #' Parse composite data frame into component data frames by variable prefix
@@ -264,10 +393,10 @@ mongo.rune <- function(collection, db_name = NULL, lower = TRUE ){
 #'
 #' @param df a dataframe containing multiple, prefixed measures
 #' @param lower default TRUE convert prefixes to lower case
-#' 
+#'
 #' @return Creates multiple dataframes in the global environment, one for each survey
 #'   detected in the data. Each dataframe is named after its survey prefix.
-#'   
+#'
 #' @details
 #' The function performs the following steps:
 #' \itemize{
@@ -296,7 +425,7 @@ mongo.rune <- function(collection, db_name = NULL, lower = TRUE ){
 #'   bar_2 = c("w", "x", "y", "z")
 #' )
 #' rune(combined_df)
-#' 
+#'
 #' # After running, access individual survey dataframes directly:
 #' head(foo)  # Access the foo dataframe
 #' head(bar)  # Access the bar dataframe
@@ -304,69 +433,69 @@ mongo.rune <- function(collection, db_name = NULL, lower = TRUE ){
 #' @importFrom dplyr filter select
 #' @export
 rune <- function(df, lower = TRUE){
-  
+
   # Define potential identifiers
   identifiers <- c("participantId", "workerId", "PROLIFIC_PID", "src_subject_id")
-  
+
   # Filter to keep only existing keys in the dataframe
   existing_keys <- identifiers[identifiers %in% names(df)]
-  
+
   # Check if any identifiers exist in the dataframe
   if (length(existing_keys) == 0) {
     stop("No valid identifiers found in the dataframe.")
   }
-  
+
   # Print existing identifiers for debugging
   #print("Existing identifiers:")
   #print(existing_keys)
-  
+
   # Find the first identifier with non-NA values
   identifier <- NA
   for (key in existing_keys) {
     non_na_count <- sum(!is.na(df[[key]]))
-    
+
     # Debug print to check how many non-NA values exist in each column
     #print(paste("Checking identifier:", key, "with", non_na_count, "non-NA values"))
-    
+
     if (non_na_count > 0) {  # As long as there's at least 1 non-NA value
       identifier <- key
       break
     }
   }
-  
+
   # If no column has any non-NA values, issue a warning or stop
   if (is.na(identifier)) {
     stop(paste("No identifier found without NA values or multiple identifiers exist:", existing_keys))
   }
-  
+
   # Print the detected identifier for debugging
   message(paste("Detected identifier:", identifier))
-  
+
   # Define common columns to include if they exist
   common_columns <- c(
     "record_id", "subjectkey", "site", "phenotype", "visit", "week",
-    "state", "status", "lost_to_followup", "lost_to_follow-up", 
+    "state", "status", "lost_to_followup", "lost_to_follow-up",
     "interview_age", "interview_date"
   )
-  
+
   # Filter to only include common columns that exist in the original dataframe
   common_columns_exist <- common_columns[common_columns %in% names(df)]
-  
+
   # Exclude non-survey and specific columns from survey prefix detection
   non_survey_columns <- c(existing_keys, common_columns)
   survey_columns <- names(df)[!names(df) %in% non_survey_columns & grepl("_", names(df))]
-  
+
   # Extract unique survey prefixes from survey-specific column names
   extract_first_part <- function(string) {
     parts <- strsplit(string, "_")[[1]]
     return(parts[1])
   }
   survey_prefixes <- unique(sapply(survey_columns, extract_first_part))
-  
+
   # Exclude prefixes that might still be problematic
   excluded_prefixes <- c("PROLIFIC", "interview")
   survey_prefixes <- survey_prefixes[!survey_prefixes %in% excluded_prefixes]
-  
+
   # Create a list of dataframes, one for each survey
   output = list()
   for (prefix in survey_prefixes) {
@@ -375,21 +504,21 @@ rune <- function(df, lower = TRUE){
       # Create subset dataframe with identifier, common columns, and survey-specific columns
       all_columns <- c(identifier, common_columns_exist, survey_specific_columns)
       subset_df <- df[, all_columns, drop = FALSE]
-      
+
       # Apply lowercase transformation if requested
       if (lower) {
         # Always preserve the identifier column name and common columns
         cols_to_transform <- !names(subset_df) %in% c(identifier, common_columns_exist)
         names(subset_df)[cols_to_transform] <- tolower(names(subset_df)[cols_to_transform])
       }
-      
+
       # Add to output list with lowercase prefix as key
       output[[tolower(prefix)]] <- subset_df
     }
   }
-  
+
   names(output) <- tolower(survey_prefixes)
-  
+
   # Use parent.frame() instead of globalenv() for CRAN compliance
   list2env(output, parent.frame())
 }
